@@ -1,7 +1,7 @@
-/* 
+/* -*- mode: objc -*-
    Project: DataBasin
 
-   Copyright (C) 2008-2012 Free Software Foundation
+   Copyright (C) 2008-2018 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -31,6 +31,8 @@
 @class DBSoap;
 @class DBSoapCSV;
 @class DBLogger;
+@class Preferences;
+@class DBProgress;
 
 @interface AppController : NSObject
 {
@@ -38,25 +40,20 @@
   DBSoapCSV *dbCsv;
   DBLogger  *logger;
   NSMutableDictionary *loginDict;
+  
+  Preferences *preferences;
 
   /* fault panel */
   IBOutlet NSPanel    *faultPanel;
   IBOutlet NSTextView *faultTextView;
   
-  /* preferences */
-  IBOutlet NSPanel       *prefPanel;
-  IBOutlet NSPopUpButton *popupStrEncoding;
-  IBOutlet NSButton      *buttPrefCancel;
-  IBOutlet NSButton      *buttPrefOk;
-  IBOutlet NSButton      *buttPrefHttps;
-  IBOutlet NSPopUpButton *popupLogLevel;
-  
   /* login */
   IBOutlet NSWindow      *winLogin;
   IBOutlet NSTextField   *fieldUserName;
   IBOutlet NSTextField   *fieldPassword;
   IBOutlet NSTextField   *fieldToken;
   IBOutlet NSPopUpButton *popupEnvironment;
+  IBOutlet NSImageView   *loginStatus;
   
   /* session status */
   IBOutlet NSWindow      *winSessionInspector;
@@ -74,13 +71,18 @@
   IBOutlet NSTextField   *fieldUserId;
   IBOutlet NSTextField   *fieldProfileId;
   IBOutlet NSTextField   *fieldRoleId;
-  
 
   /* query */
   IBOutlet NSWindow      *winSelect;
   IBOutlet NSTextView    *fieldQuerySelect;
   IBOutlet NSTextField   *fieldFileSelect;
   IBOutlet NSButton      *queryAllSelect;
+  IBOutlet NSProgressIndicator *progIndSelect;
+  IBOutlet NSTextField   *fieldRTSelect;
+  IBOutlet NSButton      *orderedWritingSelect;
+  IBOutlet NSButton      *buttonSelectExec;
+  IBOutlet NSButton      *buttonSelectStop;
+  DBProgress *selectProgress;
 
   /* query identify */
   IBOutlet NSWindow      *winSelectIdentify;
@@ -89,16 +91,45 @@
   IBOutlet NSTextField   *fieldFileSelectIdentifyOut;
   IBOutlet NSButton      *queryAllSelectIdentify;
   IBOutlet NSPopUpButton *popupBatchSizeIdentify;
+  IBOutlet NSProgressIndicator *progIndSelectIdent;
+  IBOutlet NSTextField   *fieldRTSelectIdent;
+  IBOutlet NSButton      *orderedWritingSelectIdent;
+  IBOutlet NSButton      *buttonSelectIdentExec;
+  IBOutlet NSButton      *buttonSelectIdentStop;
+  DBProgress *selectIdentProgress;
+
+  /* query retrieve */
+  IBOutlet NSWindow      *winRetrieve;
+  IBOutlet NSTextView    *fieldQueryRetrieve;
+  IBOutlet NSTextField   *fieldFileRetrieveIn;
+  IBOutlet NSTextField   *fieldFileRetrieveOut;
+  IBOutlet NSPopUpButton *popupBatchSizeRetrieve;
+  IBOutlet NSProgressIndicator *progIndRetrieve;
+  IBOutlet NSTextField   *fieldRTRetrieve;
+  IBOutlet NSButton      *orderedWritingRetrieve;
+  IBOutlet NSButton      *buttonRetrieveExec;
+  IBOutlet NSButton      *buttonRetrieveStop;
+  DBProgress *retrieveProgress;
   
   /* insert */
   IBOutlet NSWindow      *winInsert;
   IBOutlet NSTextField   *fieldFileInsert;
   IBOutlet NSPopUpButton *popupObjectsInsert;
+  IBOutlet NSProgressIndicator *progIndInsert;
+  IBOutlet NSTextField   *fieldRTInsert;
+  IBOutlet NSButton      *buttonInsertExec;
+  IBOutlet NSButton      *buttonInsertStop;
+  DBProgress *insertProgress;
 
   /* update */
   IBOutlet NSWindow      *winUpdate;
   IBOutlet NSTextField   *fieldFileUpdate;
   IBOutlet NSPopUpButton *popupObjectsUpdate;
+  IBOutlet NSProgressIndicator *progIndUpdate;
+  IBOutlet NSTextField   *fieldRTUpdate;
+  IBOutlet NSButton      *buttonUpdateExec;
+  IBOutlet NSButton      *buttonUpdateStop;
+  DBProgress *updateProgress;
 
   /* describe */
   IBOutlet NSWindow      *winDescribe;
@@ -109,18 +140,22 @@
   IBOutlet NSWindow      *winQuickDelete;
   IBOutlet NSTextField   *fieldObjectIdQd;
   IBOutlet NSTextField   *fieldStatusQd;
+  IBOutlet NSProgressIndicator *progIndDelete;
+  IBOutlet NSTextField   *fieldRTDelete;  
+  IBOutlet NSButton      *buttonQuickDeleteExec;
   
   /* mass delete */
   IBOutlet NSWindow      *winDelete;
   IBOutlet NSTextField   *fieldFileDelete;
   IBOutlet NSButton      *checkSkipFirstLine;
+  IBOutlet NSButton      *buttonDeleteExec;
+  IBOutlet NSButton      *buttonDeleteStop;
+  DBProgress *deleteProgress;
   
   /* object inspector */
   DBObjectInspector *objInspector;
 }
 
-+ (void)initialize;
-
 - (id)init;
 - (void)dealloc;
 
@@ -131,9 +166,10 @@
 - (void)applicationWillTerminate:(NSNotification *)aNotif;
 - (BOOL)application:(NSApplication *)application openFile:(NSString *)fileName;
 
+/* reload defaults that are not queryied dymanically and need be reloaded on their change */
+- (void)reloadDefaults;
+
 - (IBAction)showPrefPanel:(id)sender;
-- (IBAction)prefPanelCancel:(id)sender;
-- (IBAction)prefPanelOk:(id)sender;
 
 - (IBAction)showLogin:(id)sender;
 - (IBAction)usernameFieldAction:(id)sender;
@@ -148,11 +184,19 @@
 - (IBAction)showSelect:(id)sender;
 - (IBAction)browseFileSelect:(id)sender;
 - (IBAction)executeSelect:(id)sender;
+- (IBAction)stopSelect:(id)sender;
 
 - (IBAction)showSelectIdentify:(id)sender;
 - (IBAction)browseFileSelectIdentifyIn:(id)sender;
 - (IBAction)browseFileSelectIdentifyOut:(id)sender;
 - (IBAction)executeSelectIdentify:(id)sender;
+- (IBAction)stopSelectIdentify:(id)sender;
+
+- (IBAction)showRetrieve:(id)sender;
+- (IBAction)browseFileRetrieveIn:(id)sender;
+- (IBAction)browseFileRetrieveOut:(id)sender;
+- (IBAction)executeRetrieve:(id)sender;
+- (IBAction)stopRetrieve:(id)sender;
 
 - (IBAction)showQuickDelete:(id)sender;
 - (IBAction)quickDelete:(id)sender;
@@ -160,10 +204,12 @@
 - (IBAction)showInsert:(id)sender;
 - (IBAction)browseFileInsert:(id)sender;
 - (IBAction)executeInsert:(id)sender;
+- (IBAction)stopInsert:(id)sender;
 
 - (IBAction)showUpdate:(id)sender;
 - (IBAction)browseFileUpdate:(id)sender;
 - (IBAction)executeUpdate:(id)sender;
+- (IBAction)stopUpdate:(id)sender;
 
 - (IBAction)showDescribe:(id)sender;
 - (IBAction)browseFileDescribe:(id)sender;
@@ -172,6 +218,7 @@
 - (IBAction)showDelete:(id)sender;
 - (IBAction)browseFileDelete:(id)sender;
 - (IBAction)executeDelete:(id)sender;
+- (IBAction)stopDelete:(id)sender;
 
 - (IBAction)showObjectInspector:(id)sender;
 
