@@ -1,7 +1,7 @@
 /* 
    Project: DataBasin
 
-   Copyright (C) 2008-2016 Free Software Foundation
+   Copyright (C) 2008-2017 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -27,7 +27,9 @@
 #import "AppController.h"
 #import <DataBasinKit/DBSoap.h>
 #import <DataBasinKit/DBSoapCSV.h>
+#import <DataBasinKit/DBFileWriter.h>
 #import <DataBasinKit/DBCSVWriter.h>
+#import <DataBasinKit/DBHTMLWriter.h>
 #import <DataBasinKit/DBCSVReader.h>
 #import "DBLogger.h"
 #import "DBProgress.h"
@@ -61,13 +63,6 @@
       else
 	loginDict = [NSMutableDictionary dictionaryWithDictionary:loginDict];
       [loginDict retain];
-
-      /* if none found, set a reasonable default for the upload and insert batch size */
-      if (![defaults objectForKey:@"UpBatchSize"])
-	[defaults setObject:[NSNumber numberWithInt:100] forKey:@"UpBatchSize"];
-      /* if none found, set a reasonable default for the query batch size */
-      if (![defaults objectForKey:@"DownBatchSize"])
-	[defaults setObject:[NSNumber numberWithInt:500] forKey:@"DownBatchSize"];
     }
   return self;
 }
@@ -109,10 +104,10 @@
 {
   NSUserDefaults *defaults;
   id obj;
-  int size;
+  unsigned size;
   
   defaults = [NSUserDefaults standardUserDefaults];
-  
+
   obj = [defaults objectForKey: @"LogLevel"];
   /* if the log level is not set we set it to the standard level */
   if (obj == nil)
@@ -124,9 +119,9 @@
 
   obj = [defaults objectForKey: @"StringEncoding"];
   if (obj == nil)
-	{
-	  obj = [NSNumber numberWithInt: NSUTF8StringEncoding];
-	  [defaults setObject:obj forKey: @"StringEncoding"];
+    {
+      obj = [NSNumber numberWithInt: NSUTF8StringEncoding];
+      [defaults setObject:obj forKey: @"StringEncoding"];
     }
 	
   size = [defaults integerForKey:@"UpBatchSize"];
@@ -145,13 +140,23 @@
     }
   [db setDownBatchSize:size];
 
+  size = [defaults integerForKey:@"MaxSOQLQueryLength"];
+  if (size == 0)
+    {
+      size = MAX_SOQL_LENGTH;
+      [defaults setInteger:size forKey:@"MaxSOQLQueryLength"];
+    }
+  [db setMaxSOQLLength:size];
+
+  [db setEnableFieldTypesDescribeForQuery:[[defaults objectForKey:@"DescribeFieldTypesInQueries"] boolValue]];
+
+  [defaults synchronize];
+
   // FIXME here we should set the defaults of the CSV reader/writers
 }
 
 - (IBAction)showPrefPanel:(id)sender
 {
-  NSUserDefaults *defaults;
-  
   if (!preferences)
     {
       preferences = [[Preferences alloc] init];
@@ -229,6 +234,35 @@
   NSDictionary *uInfo;
   NSMutableDictionary *loginSet;
   NSUserDefaults *defaults;
+  BOOL useHTTPS;
+  
+  useHTTPS = YES;
+  if ([[NSProcessInfo processInfo] operatingSystem] == NSMACHOperatingSystem)
+    {
+      NSString *v;
+      NSRange rangeOf10;
+    
+      v = [[NSProcessInfo processInfo] operatingSystemVersionString];
+      NSLog(@"OS Version: %@", v);
+      rangeOf10 = [v rangeOfString:@" 10."];
+      if (rangeOf10.location != NSNotFound)
+        {
+          NSRange rangeOfDot;
+        
+          rangeOfDot = [v rangeOfString:@"." options:0 range:NSMakeRange(rangeOf10.location+rangeOf10.length, [v length]-(rangeOf10.location+rangeOf10.length))];
+          if (rangeOfDot.location != NSNotFound)
+            {
+              NSString *minorStr;
+              int minor;
+              
+              minorStr = [v substringWithRange:NSMakeRange(rangeOf10.location + rangeOf10.length, rangeOfDot.location - (rangeOf10.location + rangeOf10.length))];
+              minor = [minorStr intValue];
+              if (minor > 1 && minor < 10)
+                useHTTPS = NO;
+              NSLog(@"minor version %@: %d", minorStr, minor);
+            }
+        }
+    }
   
   userName = [fieldUserName stringValue];
   password = [fieldPassword stringValue];
@@ -239,11 +273,13 @@
   /* if present, we append the security token to the password */
   if (token != nil)
     password = [password stringByAppendingString:token];
-    
+
   db = [[DBSoap alloc] init];
   [db setLogger: logger];
   [db setUpBatchSize:[[defaults objectForKey:@"UpBatchSize"] intValue]];
   [db setDownBatchSize:[[defaults objectForKey:@"DownBatchSize"] intValue]];
+  [db setMaxSOQLLength:[[defaults objectForKey:@"MaxSOQLQueryLength"] intValue]];
+  [db setEnableFieldTypesDescribeForQuery:[[defaults objectForKey:@"DescribeFieldTypesInQueries"] boolValue]];
   dbCsv = [[DBSoapCSV alloc] init];
   [dbCsv setDBSoap:db];
   
@@ -256,7 +292,7 @@
   [logger log:LogStandard :@"[AppController doLogin] Url: %@\n", [url absoluteString]];  
   
   NS_DURING
-    [db login :url :userName :password :YES];
+    [db login :url :userName :password :useHTTPS];
     
     /* session inspector fields */
     [fieldSessionId setStringValue:[db sessionId]];
@@ -279,7 +315,7 @@
 
   NS_HANDLER
     [logger log:LogStandard :@"Login failed\n"];
-  [loginStatus setImage:[NSImage imageNamed:@"butt_red_16.tif"]];
+    [loginStatus setImage:[NSImage imageNamed:@"butt_red_16.tif"]];
     if ([[localException name] hasPrefix:@"DB"])
       {
         [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
@@ -322,13 +358,16 @@
 	    oldKey = key;
 	  currSet = [loginDict objectForKey:key];
 	  oldSet = [loginDict objectForKey:oldKey];
-	  if ([[currSet objectForKey:@"lastlogin"] compare: [oldSet objectForKey:@"lastlogin"]] == NSOrderedAscending)
+	  if ([(NSString *)[currSet objectForKey:@"lastlogin"] compare: [oldSet objectForKey:@"lastlogin"]] == NSOrderedAscending)
 	    oldKey = key;
 	}
       [logger log:LogInformative :@"[AppController doLogin] delete: %@\n", oldKey];
       [loginDict removeObjectForKey:oldKey];
     }
   [[NSUserDefaults standardUserDefaults] setObject:loginDict forKey: @"logins"];
+
+  /* set or update soap handlers */
+  [objInspector setSoapHandler:db];
 }
 
 /* UPDATE SOBJECT LIST */
@@ -357,9 +396,11 @@
 - (IBAction)browseFileSelect:(id)sender
 {
   NSSavePanel *savePanel;
+  NSArray *types;
   
+  types = [NSArray arrayWithObjects:@"csv", @"html", @"xls", nil];
   savePanel = [NSSavePanel savePanel];
-  [savePanel setRequiredFileType:@"csv"];
+  [savePanel setAllowedFileTypes:types];
   if ([savePanel runModal] == NSOKButton)
     {
       NSString *fileName;
@@ -381,15 +422,21 @@
   NSString       *filePath;
   NSFileHandle   *fileHandle;
   NSFileManager  *fileManager;
-  DBCSVWriter    *csvWriter;
+  DBFileWriter   *fileWriter;
   NSString       *str;
   NSUserDefaults *defaults;
   NSAutoreleasePool *arp;
+  NSString       *fileType;
   
   arp = [NSAutoreleasePool new];
   defaults = [NSUserDefaults standardUserDefaults];
   statement = [fieldQuerySelect string];
   filePath = [fieldFileSelect stringValue];
+  fileType = DBFileFormatCSV;
+  if ([[[filePath pathExtension] lowercaseString] isEqualToString:@"html"])
+    fileType = DBFileFormatHTML;
+  else if ([[[filePath pathExtension] lowercaseString] isEqualToString:@"xls"])
+    fileType = DBFileFormatXLS;
   
   fileManager = [NSFileManager defaultManager];
   if ([fileManager createFileAtPath:filePath contents:nil attributes:nil] == NO)
@@ -412,31 +459,46 @@
   [selectProgress setRemainingTimeField: fieldRTSelect];
   [selectProgress setLogger:logger];
   [selectProgress reset];
-  csvWriter = [[DBCSVWriter alloc] initWithHandle:fileHandle];
-  [csvWriter setLogger:logger];
-  [csvWriter setWriteFieldsOrdered:([orderedWritingSelect state] == NSOnState)];
-  [csvWriter setLineBreakHandling:[defaults integerForKey:CSVWriteLineBreakHandling]];
-  str = [defaults stringForKey:@"CSVWriteQualifier"];
-  if (str)
-    [csvWriter setQualifier:str];
-  str = [defaults stringForKey:@"CSVWriteSeparator"];
-  if (str)
-    [csvWriter setSeparator:str];
+  fileWriter = nil;
   
+  if (fileType == DBFileFormatCSV)
+    {
+      fileWriter = [[DBCSVWriter alloc] initWithHandle:fileHandle];
+      [(DBCSVWriter *)fileWriter setLineBreakHandling:[defaults integerForKey:CSVWriteLineBreakHandling]];
+      str = [defaults stringForKey:@"CSVWriteQualifier"];
+      if (str)
+        [(DBCSVWriter *)fileWriter setQualifier:str];
+      str = [defaults stringForKey:@"CSVWriteSeparator"];
+      if (str)
+        [(DBCSVWriter *)fileWriter setSeparator:str];
+    }
+  else if (fileType == DBFileFormatHTML || fileType == DBFileFormatXLS)
+    {
+      fileWriter = [[DBHTMLWriter alloc] initWithHandle:fileHandle];
+      if (fileType == DBFileFormatXLS)
+        [fileWriter setFileFormat:DBFileFormatXLS];
+      else
+        [fileWriter setFileFormat:DBFileFormatHTML];
+    }
+  [fileWriter setWriteFieldsOrdered:([orderedWritingSelect state] == NSOnState)];
+  [fileWriter setLogger:logger];
+  [fileWriter setStringEncoding: [defaults integerForKey: @"StringEncoding"]];
+  NSLog(@"fileType is: %@, writer: %@", fileType, fileWriter);
   NS_DURING
-    [dbCsv query :statement queryAll:([queryAllSelect state] == NSOnState) toWriter:csvWriter progressMonitor:selectProgress];
+    [dbCsv query :statement queryAll:([queryAllSelect state] == NSOnState) toWriter:fileWriter progressMonitor:selectProgress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
         [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
-  [csvWriter release];
+
+  [fileWriter release];
   [fileHandle closeFile];
   [selectProgress release];
   selectProgress = nil;
   [self performSelectorOnMainThread:@selector(resetSelectUI:) withObject:self waitUntilDone:NO];
-  [arp drain];
+  [arp release];
 }
 
 - (IBAction)executeSelect:(id)sender
@@ -465,7 +527,16 @@
 - (IBAction)showInsert:(id)sender
 {
   NSArray *objectNames;
+  NSMutableArray *filteredObjectNames;
+  BOOL filterShare;
+  BOOL filterHistory;
+  NSUInteger i;
+  NSUserDefaults *defaults;
 
+  defaults = [NSUserDefaults standardUserDefaults];
+  filterShare = [defaults boolForKey:@"FilterObjects_Share"];
+  filterHistory = [defaults boolForKey:@"FilterObjects_History"];
+  
   [winInsert makeKeyAndOrderFront:self];
   [progIndInsert setIndeterminate:YES];
   objectNames = nil;
@@ -477,10 +548,26 @@
         [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
+    
+  filteredObjectNames = [[NSMutableArray alloc] initWithCapacity:1];
+  for (i = 0; i < [objectNames count]; i++)
+    {
+      NSString *name;
+
+      name = [objectNames objectAtIndex:i];
+      if (filterShare && [name hasSuffix: @"Share"])
+	name = nil;
+      if (filterHistory && [name hasSuffix: @"History"])
+	name = nil;
+
+      if (name)
+	[filteredObjectNames addObject:name];
+    }
   [popupObjectsInsert removeAllItems];
-  [popupObjectsInsert addItemsWithTitles: objectNames];
+  [popupObjectsInsert addItemsWithTitles: filteredObjectNames];
   [progIndInsert setIndeterminate:NO];
   [progIndInsert setDoubleValue:0];
+  [filteredObjectNames release];
 }
 
 - (IBAction)browseFileInsert:(id)sender
@@ -515,14 +602,17 @@
   arp = [NSAutoreleasePool new];
   defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileInsert stringValue];
+  if (filePath == nil)
+    {
+      [logger log:LogInformative :@"[AppController performInsert] nil file paths\n"];
+      return;
+    }
   resFilePath = [[filePath stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"results.csv"];
 
   NSLog(@"writing results to: %@", resFilePath);
-
-  filePath = [fieldFileInsert stringValue];
   
   intoWhichObject = [[[popupObjectsInsert selectedItem] title] retain];
-  [logger log:LogInformative :@"[AppController executeInsert] object: %@\n", intoWhichObject];
+  [logger log:LogInformative :@"[AppController performInsert] object: %@\n", intoWhichObject];
 
   insertProgress = [[DBProgress alloc] init];
   [insertProgress setLogger:logger];
@@ -547,15 +637,13 @@
     if ([[localException name] hasPrefix:@"DB"])
       {
         [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
-        [insertProgress release];
         [self resetInsertUI:self];
-        [arp drain];
       }
   NS_ENDHANDLER
 
 
   fileManager = [NSFileManager defaultManager];
-  if ([fileManager createFileAtPath:resFilePath contents:nil attributes:nil] == NO)
+  if (results && [fileManager createFileAtPath:resFilePath contents:nil attributes:nil] == NO)
     {
       NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
     }  
@@ -597,7 +685,7 @@
   insertProgress = nil;
   [self performSelectorOnMainThread:@selector(resetInsertUI:) withObject:self waitUntilDone:NO];
   [results release];
-  [arp drain];
+  [arp release];
 }
 
 - (IBAction)executeInsert:(id)sender
@@ -618,7 +706,16 @@
 - (IBAction)showUpdate:(id)sender
 {
   NSArray      *objectNames;
-  
+  NSMutableArray *filteredObjectNames;
+  BOOL filterShare;
+  BOOL filterHistory;
+  NSUInteger i;
+  NSUserDefaults *defaults;
+
+  defaults = [NSUserDefaults standardUserDefaults];
+  filterShare = [defaults boolForKey:@"FilterObjects_Share"];
+  filterHistory = [defaults boolForKey:@"FilterObjects_History"];
+
   [winUpdate makeKeyAndOrderFront:self];
   [progIndUpdate setIndeterminate:YES];
   objectNames  = nil;
@@ -630,10 +727,27 @@
         [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
+
+  filteredObjectNames = [[NSMutableArray alloc] initWithCapacity:1];
+  for (i = 0; i < [objectNames count]; i++)
+    {
+      NSString *name;
+
+      name = [objectNames objectAtIndex:i];
+      if (filterShare && [name hasSuffix: @"Share"])
+	name = nil;
+      if (filterHistory && [name hasSuffix: @"History"])
+	name = nil;
+
+      if (name)
+	[filteredObjectNames addObject:name];
+    }
+  
   [popupObjectsUpdate removeAllItems];
-  [popupObjectsUpdate addItemsWithTitles: objectNames];
+  [popupObjectsUpdate addItemsWithTitles: filteredObjectNames];
   [progIndUpdate setIndeterminate:NO];
-  [progIndUpdate setDoubleValue:0]; 
+  [progIndUpdate setDoubleValue:0];
+  [filteredObjectNames release];
 }
 
 - (IBAction)browseFileUpdate:(id)sender
@@ -755,7 +869,7 @@
   updateProgress = nil;
   [results release];
   [self performSelectorOnMainThread:@selector(resetUpdateUI:) withObject:self waitUntilDone:NO];
-  [arp drain];
+  [arp release];
 }
 
 - (IBAction)executeUpdate:(id)sender
@@ -797,9 +911,11 @@
 - (IBAction)browseFileSelectIdentifyOut:(id)sender
 {
   NSSavePanel *savePanel;
+  NSArray *types;
   
+  types = [NSArray arrayWithObjects:@"csv", @"html", @"xls", nil];  
   savePanel = [NSSavePanel savePanel];
-  [savePanel setRequiredFileType:@"csv"];
+  [savePanel setAllowedFileTypes:types];
   if ([savePanel runModal] == NSOKButton)
     {
       NSString *fileName;
@@ -822,8 +938,9 @@
   NSString       *filePathOut;
   NSFileHandle   *fileHandleOut;
   NSFileManager  *fileManager;
-  DBCSVWriter    *csvWriter;
+  DBFileWriter   *fileWriter;
   DBCSVReader    *csvReader;
+  NSString       *fileTypeOut;
   int            batchSize;
   NSString       *str;
   NSUserDefaults *defaults;
@@ -835,12 +952,17 @@
   statement = [fieldQuerySelectIdentify string];
   filePathIn = [fieldFileSelectIdentifyIn stringValue];
   filePathOut = [fieldFileSelectIdentifyOut stringValue];
-
+  fileTypeOut = DBFileFormatCSV;
+  if ([[[filePathOut pathExtension] lowercaseString] isEqualToString:@"html"])
+    fileTypeOut = DBFileFormatHTML;
+  else if ([[[filePathOut pathExtension] lowercaseString] isEqualToString:@"xls"])
+    fileTypeOut = DBFileFormatXLS;
+  
   batchSize = 0;
   switch ([[popupBatchSizeIdentify selectedItem] tag])
     {
     case 1:
-      batchSize = 0;
+      batchSize = 1;
       break;
     case 2:
       batchSize = 10;
@@ -876,7 +998,7 @@
     {
       NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
       [csvReader release];
-      [arp drain];
+      [arp release];
       [self performSelectorOnMainThread:@selector(resetSelectIdentUI:) withObject:self waitUntilDone:NO];
       return;
     }  
@@ -886,21 +1008,37 @@
     {
       NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
       [csvReader release];
-      [arp drain];
+      [arp release];
       [self performSelectorOnMainThread:@selector(resetSelectIdentUI:) withObject:self waitUntilDone:NO];
       return;
     }
 
-  csvWriter = [[DBCSVWriter alloc] initWithHandle:fileHandleOut];
-  [csvWriter setLogger:logger];
-  [csvWriter setWriteFieldsOrdered:([orderedWritingSelectIdent state] == NSOnState)];
-  str = [defaults stringForKey:@"CSVWriteQualifier"];
-  if (str)
-    [csvWriter setQualifier:str];
-  str = [defaults stringForKey:@"CSVWriteSeparator"];
-  if (str)
-    [csvWriter setSeparator:str];
-  [csvWriter setLineBreakHandling:[defaults integerForKey:CSVWriteLineBreakHandling]];
+  fileWriter = nil;
+  if (fileTypeOut == DBFileFormatCSV)
+    {
+      fileWriter = [[DBCSVWriter alloc] initWithHandle:fileHandleOut];
+      str = [defaults stringForKey:@"CSVWriteQualifier"];
+      if (str)
+	[(DBCSVWriter *)fileWriter setQualifier:str];
+      str = [defaults stringForKey:@"CSVWriteSeparator"];
+      if (str)
+	[(DBCSVWriter *)fileWriter setSeparator:str];
+      [(DBCSVWriter *)fileWriter setLineBreakHandling:[defaults integerForKey:CSVWriteLineBreakHandling]];
+    }
+  else if (fileTypeOut == DBFileFormatHTML || fileTypeOut == DBFileFormatXLS)
+    {
+      fileWriter = [[DBHTMLWriter alloc] initWithHandle:fileHandleOut];
+      if (fileTypeOut == DBFileFormatXLS)
+        [fileWriter setFileFormat:DBFileFormatXLS];
+      else
+        [fileWriter setFileFormat:DBFileFormatHTML];
+    }
+
+  [fileWriter setLogger:logger];
+  [fileWriter setWriteFieldsOrdered:([orderedWritingSelectIdent state] == NSOnState)];
+  [fileWriter setStringEncoding: [defaults integerForKey: @"StringEncoding"]];
+
+  
   selectIdentProgress = [[DBProgress alloc] init];
   [selectIdentProgress setLogger:logger];
   [selectIdentProgress setProgressIndicator: progIndSelectIdent];
@@ -908,7 +1046,7 @@
   [selectIdentProgress reset];
 
   NS_DURING
-    [dbCsv queryIdentify :statement queryAll:([queryAllSelectIdentify state] == NSOnState) fromReader:csvReader toWriter:csvWriter withBatchSize:batchSize progressMonitor:selectIdentProgress];
+    [dbCsv queryIdentify :statement queryAll:([queryAllSelectIdentify state] == NSOnState) fromReader:csvReader toWriter:fileWriter withBatchSize:batchSize progressMonitor:selectIdentProgress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -917,13 +1055,13 @@
   NS_ENDHANDLER
 
   [csvReader release];
-  [csvWriter release];
+  [fileWriter release];
   [fileHandleOut closeFile];
   
   [selectIdentProgress release];
   selectIdentProgress = nil;
   [self performSelectorOnMainThread:@selector(resetSelectIdentUI:) withObject:self waitUntilDone:NO];
-  [arp drain];
+  [arp release];
 }
 
 - (IBAction)executeSelectIdentify:(id)sender
@@ -939,6 +1077,206 @@
   [selectIdentProgress setShouldStop:YES];
 }
 
+
+/*  RETRIEVE  */
+
+- (IBAction)showRetrieve:(id)sender
+{
+  [winRetrieve makeKeyAndOrderFront:self];
+  [progIndRetrieve setIndeterminate:NO];
+  [progIndRetrieve setDoubleValue:0];
+}
+
+- (IBAction)browseFileRetrieveIn:(id)sender
+{
+  NSOpenPanel *openPanel;
+  
+  openPanel = [NSOpenPanel openPanel];
+  if ([openPanel runModal] == NSOKButton)
+    {
+      NSString *fileName;
+      
+      fileName = [openPanel filename];
+      [fieldFileRetrieveIn setStringValue:fileName];
+    }
+}
+
+- (IBAction)browseFileRetrieveOut:(id)sender
+{
+  NSSavePanel *savePanel;
+  NSArray *types;
+  
+  types = [NSArray arrayWithObjects:@"csv", @"html", @"xls", nil];  
+  savePanel = [NSSavePanel savePanel];
+  [savePanel setAllowedFileTypes:types];
+  if ([savePanel runModal] == NSOKButton)
+    {
+      NSString *fileName;
+      
+      fileName = [savePanel filename];
+      [fieldFileRetrieveOut setStringValue:fileName];
+    }
+}
+
+- (void)resetRetrieveUI:(id)arg
+{
+  [buttonRetrieveExec setEnabled:YES];
+  [buttonRetrieveStop setEnabled:NO];
+}
+
+- (void)performRetrieve:(id)arg
+{
+  NSString       *statement;
+  NSString       *filePathIn;
+  NSString       *filePathOut;
+  NSFileHandle   *fileHandleOut;
+  NSFileManager  *fileManager;
+  DBFileWriter   *fileWriter;
+  DBCSVReader    *csvReader;
+  NSString       *fileTypeOut;
+  int            batchSize;
+  NSString       *str;
+  NSUserDefaults *defaults;
+  NSAutoreleasePool *arp;
+
+  arp = [NSAutoreleasePool new];
+  defaults = [NSUserDefaults standardUserDefaults];
+
+  statement = [fieldQueryRetrieve string];
+  filePathIn = [fieldFileRetrieveIn stringValue];
+  filePathOut = [fieldFileRetrieveOut stringValue];
+  fileTypeOut = DBFileFormatCSV;
+  if ([[[filePathOut pathExtension] lowercaseString] isEqualToString:@"html"])
+    fileTypeOut = DBFileFormatHTML;
+  else if ([[[filePathOut pathExtension] lowercaseString] isEqualToString:@"xls"])
+    fileTypeOut = DBFileFormatXLS;
+  
+  batchSize = 0;
+  switch ([[popupBatchSizeRetrieve selectedItem] tag])
+    {
+    case 1:
+      batchSize = 1;
+      break;
+    case 2:
+      batchSize = 10;
+      break;
+    case 3:
+      batchSize = 50;
+      break;
+    case 4:
+      batchSize = 100;
+      break;      
+    case 5:
+      batchSize = 200;
+      break;
+    case 6:
+      batchSize = 500;
+      break;
+    case 7:
+      batchSize = 1000;
+      break;
+    case 99:
+      batchSize = -1;
+      break;
+    default:
+      [logger log:LogStandard :@"[AppController executeRetrieve] unexpected batch size\n"];
+    }
+  [logger log:LogDebug :@"[AppController executeRetrieve] batch Size: %d\n", batchSize];
+  
+  fileManager = [NSFileManager defaultManager];
+
+  csvReader = [[DBCSVReader alloc] initWithPath:filePathIn withLogger:logger];
+  str = [defaults stringForKey:@"CSVReadQualifier"];
+  if (str)
+    [csvReader setQualifier:str];
+  str = [defaults stringForKey:@"CSVReadSeparator"];
+  if (str)
+    [csvReader setSeparator:str];
+  [csvReader parseHeaders];
+  if ([fileManager createFileAtPath:filePathOut contents:nil attributes:nil] == NO)
+    {
+      NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
+      [csvReader release];
+      [arp release];
+      [self performSelectorOnMainThread:@selector(resetRetrieveUI:) withObject:self waitUntilDone:NO];
+      return;
+    }  
+
+  fileHandleOut = [NSFileHandle fileHandleForWritingAtPath:filePathOut];
+  if (fileHandleOut == nil)
+    {
+      NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
+      [csvReader release];
+      [arp release];
+      [self performSelectorOnMainThread:@selector(resetRetrieveUI:) withObject:self waitUntilDone:NO];
+      return;
+    }
+
+  fileWriter = nil;
+  if (fileTypeOut == DBFileFormatCSV)
+    {
+      fileWriter = [[DBCSVWriter alloc] initWithHandle:fileHandleOut];
+      str = [defaults stringForKey:@"CSVWriteQualifier"];
+      if (str)
+	[(DBCSVWriter *)fileWriter setQualifier:str];
+      str = [defaults stringForKey:@"CSVWriteSeparator"];
+      if (str)
+	[(DBCSVWriter *)fileWriter setSeparator:str];
+      [(DBCSVWriter *)fileWriter setLineBreakHandling:[defaults integerForKey:CSVWriteLineBreakHandling]];
+    }
+  else if (fileTypeOut == DBFileFormatHTML || fileTypeOut == DBFileFormatXLS)
+    {
+      fileWriter = [[DBHTMLWriter alloc] initWithHandle:fileHandleOut];
+      if (fileTypeOut == DBFileFormatXLS)
+        [fileWriter setFileFormat:DBFileFormatXLS];
+      else
+        [fileWriter setFileFormat:DBFileFormatHTML];
+    }
+
+  [fileWriter setLogger:logger];
+  [fileWriter setWriteFieldsOrdered:([orderedWritingRetrieve state] == NSOnState)];
+  [fileWriter setStringEncoding: [defaults integerForKey: @"StringEncoding"]];
+
+  
+  retrieveProgress = [[DBProgress alloc] init];
+  [retrieveProgress setLogger:logger];
+  [retrieveProgress setProgressIndicator: progIndRetrieve];
+  [retrieveProgress setRemainingTimeField: fieldRTRetrieve];
+  [retrieveProgress reset];
+
+  NS_DURING
+    [dbCsv retrieve :statement fromReader:csvReader toWriter:fileWriter withBatchSize:batchSize progressMonitor:retrieveProgress];
+  NS_HANDLER
+    if ([[localException name] hasPrefix:@"DB"])
+      {
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
+      }
+  NS_ENDHANDLER
+
+  [csvReader release];
+  [fileWriter release];
+  [fileHandleOut closeFile];
+  
+  [retrieveProgress release];
+  retrieveProgress = nil;
+  [self performSelectorOnMainThread:@selector(resetRetrieveUI:) withObject:self waitUntilDone:NO];
+  [arp release];
+}
+
+- (IBAction)executeRetrieve:(id)sender
+{
+  [buttonRetrieveExec setEnabled:NO];
+  [buttonRetrieveStop setEnabled:YES];
+  [NSThread detachNewThreadSelector:@selector(performRetrieve:) toTarget:self withObject:nil];
+}
+
+
+- (IBAction)stopRetrieve:(id)sender
+{
+  [retrieveProgress setShouldStop:YES];
+}
+
+
 /* DESCRIBE */
 
 - (IBAction)showDescribe:(id)sender
@@ -946,6 +1284,8 @@
   NSArray *objectNames;
 
   objectNames = [db sObjectNames];
+  [logger log:LogStandard :@"[AppController showDescribe] Objects: %lu", (unsigned long)[objectNames count]];
+
   [popupObjectsDescribe removeAllItems];
   [popupObjectsDescribe addItemsWithTitles: objectNames];
     
@@ -955,9 +1295,11 @@
 - (IBAction)browseFileDescribe:(id)sender
 {
   NSSavePanel *savePanel;
-  
+  NSArray *types;
+
+  types = [NSArray arrayWithObjects:@"csv", @"html", @"xls", nil];
   savePanel = [NSSavePanel savePanel];
-  [savePanel setRequiredFileType:@"csv"];
+  [savePanel setAllowedFileTypes:types];
 
   if ([savePanel runModal] == NSOKButton)
     {
@@ -971,18 +1313,23 @@
 - (IBAction)executeDescribe:(id)sender
 {
   NSString       *filePath;
-  DBCSVWriter    *writer;
+  DBFileWriter    *writer;
   NSString       *whichObject;
   NSFileManager  *fileManager;
   NSFileHandle   *fileHandle;
   NSUserDefaults *defaults;
   NSString       *str;
+  NSString       *fileType;
 
   defaults = [NSUserDefaults standardUserDefaults];
     
   filePath = [fieldFileDescribe stringValue];
-  NSLog(@"%@", filePath);
-
+  fileType = DBFileFormatCSV;
+  if ([[[filePath pathExtension] lowercaseString] isEqualToString:@"html"])
+    fileType = DBFileFormatHTML;
+  else if ([[[filePath pathExtension] lowercaseString] isEqualToString:@"xls"])
+    fileType = DBFileFormatXLS;
+  
   fileManager = [NSFileManager defaultManager];
   if ([fileManager createFileAtPath:filePath contents:nil attributes:nil] == NO)
     {
@@ -995,19 +1342,32 @@
     {
       NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
     }
+
+  writer = nil;
+  if (fileType == DBFileFormatCSV)
+    {
+      writer = [[DBCSVWriter alloc] initWithHandle:fileHandle];
+      [(DBCSVWriter *)writer setLineBreakHandling:[defaults integerForKey:CSVWriteLineBreakHandling]];
+      str = [defaults stringForKey:@"CSVWriteQualifier"];
+      if (str)
+        [(DBCSVWriter *)writer setQualifier:str];
+      str = [defaults stringForKey:@"CSVWriteSeparator"];
+      if (str)
+        [(DBCSVWriter *)writer setSeparator:str];
+    }
+  else if (fileType == DBFileFormatHTML || fileType == DBFileFormatXLS)
+    {
+      writer = [[DBHTMLWriter alloc] initWithHandle:fileHandle];
+      if (fileType == DBFileFormatXLS)
+        [writer setFileFormat:DBFileFormatXLS];
+      else
+        [writer setFileFormat:DBFileFormatHTML];
+    }
   
-  writer = [[DBCSVWriter alloc] initWithHandle:fileHandle];
   [writer setLogger:logger];
   [writer setStringEncoding: [defaults integerForKey: @"StringEncoding"]];
-  str = [defaults stringForKey:@"CSVWriteQualifier"];
-  if (str)
-    [writer setQualifier:str];
-  str = [defaults stringForKey:@"CSVWriteSeparator"];
-  if (str)
-    [writer setSeparator:str];
-  
+
   whichObject = [[[popupObjectsDescribe selectedItem] title] retain];
-  NSLog(@"object: %@", whichObject);
   
   NS_DURING
     [dbCsv describeSObject:whichObject toWriter:writer];
@@ -1017,8 +1377,9 @@
         [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
-    
+
   [writer release];
+  [fileHandle closeFile];
   [whichObject release];
 }
 
@@ -1052,7 +1413,10 @@
   objectId = [fieldObjectIdQd stringValue];
   
   if (objectId == nil || [objectId length] == 0)
-    return;
+    {
+      [arp release];
+      return;
+    }
 
   /* we clone the soap instance and pass the session, so that the method can run in a separate thread */
   dbSoap = [[DBSoap alloc] init];
@@ -1073,35 +1437,26 @@
       }
   NS_ENDHANDLER
 
-    if ([resultArray count] > 0)
-      {
-        NSDictionary *resultDict;
-        NSString     *resultMsgStr;
+  if ([resultArray count] > 0)
+    {
+      NSDictionary *resultDict;
+      NSString     *resultMsgStr;
       
-        resultDict = [resultArray objectAtIndex:0];
-        if ([[resultDict objectForKey:@"success"] isEqualToString:@"true"])
-          [fieldStatusQd setStringValue:@"Deletion completed."];
-        else
-          {
-            resultMsgStr = [resultDict objectForKey:@"message"];
-            [fieldStatusQd setStringValue:[resultDict objectForKey:@"statusCode"]];
-            [faultTextView setString:resultMsgStr];
-            [faultPanel makeKeyAndOrderFront:nil];
-          }
-      }
-    else
-      {
-        NSDictionary *resultDict;
-        NSString     *resultMsgStr;
+      resultDict = [resultArray objectAtIndex:0];
+      if ([[resultDict objectForKey:@"success"] isEqualToString:@"true"])
+        [fieldStatusQd setStringValue:@"Deletion completed."];
+      else
+        {
+          resultMsgStr = [resultDict objectForKey:@"message"];
+          [fieldStatusQd setStringValue:[resultDict objectForKey:@"statusCode"]];
+          [faultTextView setString:resultMsgStr];
+          [faultPanel makeKeyAndOrderFront:nil];
+        }
+    }
 
-        resultDict = [resultArray objectAtIndex:0];
-        resultMsgStr = [resultDict objectForKey:@"message"];
-        [fieldStatusQd setStringValue:[resultDict objectForKey:@"statusCode"]];
-        [faultTextView setString:resultMsgStr];
-        [faultPanel makeKeyAndOrderFront:nil];
-      }
-  [self performSelectorOnMainThread:@selector(resetQuickDeleteUI:) withObject:self waitUntilDone:NO];    
-  [arp drain];
+  [self performSelectorOnMainThread:@selector(resetQuickDeleteUI:) withObject:self waitUntilDone:NO];
+  [dbSoap release];
+  [arp release];
 }
 
 - (IBAction)quickDelete:(id)sender
@@ -1219,7 +1574,7 @@
   [deleteProgress release];
   deleteProgress = nil;
   [self performSelectorOnMainThread:@selector(resetDeleteUI:) withObject:self waitUntilDone:NO];
-  [arp drain];
+  [arp release];
 }
 
 - (IBAction)executeDelete:(id)sender
@@ -1238,7 +1593,6 @@
 
 - (IBAction)showObjectInspector:(id)sender
 {
-  [objInspector setSoapHandler: db];
   [objInspector show];
 }
 
