@@ -1,7 +1,7 @@
 /*
  Project: DataBasin
  
- Copyright (C) 2010-2014 Free Software Foundation
+ Copyright (C) 2010-2016 Free Software Foundation
  
  Author: Riccardo Mottola
  
@@ -41,15 +41,24 @@
   IBOutlet NSTableView *fieldTable;
   IBOutlet NSTextView *faultTextView;
   IBOutlet NSPanel *faultPanel;
+  IBOutlet NSButton *loadButton;
+  IBOutlet NSButton *updateButton;
+  IBOutlet NSSearchField *searchField;
+  IBOutlet NSTextField *statusField;
 
   /* data source objects */
   DBSObject      *sObj;
   NSMutableArray *arrayRows;
+  NSMutableArray *filteredRows;
+
+  NSMutableArray *updatedRows;
 }
 
 /** sets the Soap handler class, which needs to remain valid througout the inspector existence */
 - (void)setSoapHandler:(DBSoap *)db;
 - (void)show;
+- (IBAction)search:(id)sender;
 - (IBAction)loadObject:(id)sender;
+- (IBAction)updateObject:(id)sender;
 
 @end
